@@ -148,6 +148,7 @@ sub PrintInvGPSCoordinates($)
         $v[2] = Image::ExifTool::ToFloat($v[2]) * ($below ? -1 : 1) if @v == 3;
         return "@v";
     }
+    return $val if $val =~ /^([-+]?\d+(\.\d*)?)\s+([-+]?\d+(\.\d*)?)$/; # already 2 floats?
     return $val if $val =~ /^([-+]\d+(\.\d*)?){2,3}(CRS.*)?\/?$/; # already in ISO6709 format?
     return undef;
 }
@@ -356,6 +357,18 @@ sub SetVarInt($$)
     return '';
 }
 
+#------------------------------------------------------------------------------
+# Write Nextbase infi atom (ref PH)
+# Inputs: 0) ExifTool object ref, 1) dirInfo ref, 2) tag table ref
+# Returns: updated infi data
+sub WriteNextbase($$$)
+{
+    my ($et, $dirInfo, $tagTablePtr) = @_;
+    $et or return 1;
+    $$et{DEL_GROUP}{Nextbase} and ++$$et{CHANGED}, return '';
+    return ${$$dirInfo{DataPt}};
+}
+
 #------------------------------------------------------------------------------
 # Write Meta Keys to add/delete entries as necessary ('mdta' handler) (ref PH)
 # Inputs: 0) ExifTool object ref, 1) dirInfo ref, 2) tag table ref
@@ -785,7 +798,7 @@ sub WriteQuickTime($$$)
     my ($rtnVal, $rtnErr) = $dataPt ? (undef, undef) : (1, 0);
 
     if ($dataPt) {
-        $raf = new File::RandomAccess($dataPt);
+        $raf = File::RandomAccess->new($dataPt);
     } else {
         return 0 unless $raf;
     }
@@ -1619,7 +1632,8 @@ sub WriteQuickTime($$$)
         if ($isEmpty) {
             $et->VPrint(0,'  Deleting ' . join('+', sort map { $emptyMeta{$_} } keys %boxPos)) if %boxPos;
             $$outfile = '';
-            ++$$et{CHANGED};
+            # (could report a file if editing nothing when it contained an empty Meta atom)
+            # ++$$et{CHANGED};
         }
         if ($curPath eq 'MOV-Movie-Meta') {
             delete $$addDirs{Keys}; # prevent creation of another Meta for Keys tags
